#include "triple.h"

// Extern inline prototypes.
LSUP_Term *LSUP_triple_pos (const LSUP_Triple *trp, LSUP_TriplePos n);
LSUP_Buffer *LSUP_striple_pos (const LSUP_SerTriple *trp, LSUP_TriplePos n);


LSUP_Triple *
LSUP_triple_new(LSUP_Term *s, LSUP_Term *p, LSUP_Term *o)
{
    LSUP_Triple *spo = malloc (sizeof (*spo));
    if (!spo) return NULL;

    if (UNLIKELY (LSUP_triple_init (spo, s, p, o))) {
        free (spo);
        return NULL;
    }

    return spo;
}


LSUP_SerTriple *
LSUP_striple_new(LSUP_Buffer *s, LSUP_Buffer *p, LSUP_Buffer *o)
{
    LSUP_SerTriple *sspo = malloc (sizeof (*sspo));
    if (!sspo) return NULL;

    if (UNLIKELY (LSUP_striple_init (sspo, s, p, o))) {
        free (sspo);
        return NULL;
    }

    return sspo;
}


LSUP_Triple *
LSUP_triple_new_from_striple (const LSUP_SerTriple *sspo)
{
    LSUP_Triple *spo = malloc (sizeof (*spo));
    if (!spo) return NULL;

    spo->s = LSUP_term_new_from_buffer (sspo->s);
    spo->p = LSUP_term_new_from_buffer (sspo->p);
    spo->o = LSUP_term_new_from_buffer (sspo->o);

    return spo;
}


LSUP_SerTriple *
LSUP_striple_new_from_triple (const LSUP_Triple *spo)
{
    LSUP_SerTriple *sspo = malloc (sizeof (*sspo));
    if (!sspo) return NULL;

    sspo->s = LSUP_buffer_new_from_term (spo->s);
    sspo->p = LSUP_buffer_new_from_term (spo->p);
    sspo->o = LSUP_buffer_new_from_term (spo->o);

    return sspo;
}


LSUP_rc
LSUP_triple_init (LSUP_Triple *spo, LSUP_Term *s, LSUP_Term *p, LSUP_Term *o)
{
    spo->s = s;
    spo->p = p;
    spo->o = o;

    return LSUP_OK;
}


LSUP_rc
LSUP_striple_init (
        LSUP_SerTriple *sspo, LSUP_Buffer *s, LSUP_Buffer *p, LSUP_Buffer *o)
{
    sspo->s = s;
    sspo->p = p;
    sspo->o = o;

    return LSUP_OK;
}


LSUP_rc
LSUP_triple_serialize (const LSUP_Triple *spo, LSUP_SerTriple *sspo)
{
    LSUP_rc rc;

    rc = LSUP_term_serialize (spo->s, sspo->s);
    if (UNLIKELY (rc != LSUP_OK)) return rc;
    rc = LSUP_term_serialize (spo->p, sspo->p);
    if (UNLIKELY (rc != LSUP_OK)) return rc;
    rc = LSUP_term_serialize (spo->o, sspo->o);
    if (UNLIKELY (rc != LSUP_OK)) return rc;

    return LSUP_OK;
}


LSUP_rc
LSUP_triple_deserialize (const LSUP_SerTriple *sspo, LSUP_Triple *spo)
{
    LSUP_rc rc;

    if (!spo->s) spo->s = malloc (sizeof (*spo->s));
    if (!spo->p) spo->p = malloc (sizeof (*spo->p));
    if (!spo->o) spo->o = malloc (sizeof (*spo->o));

    rc = LSUP_term_deserialize (sspo->s, spo->s);
    if (UNLIKELY (rc != LSUP_OK)) return rc;
    rc = LSUP_term_deserialize (sspo->p, spo->p);
    if (UNLIKELY (rc != LSUP_OK)) return rc;
    rc = LSUP_term_deserialize (sspo->o, spo->o);
    if (UNLIKELY (rc != LSUP_OK)) return rc;

    return LSUP_OK;
}


void
LSUP_triple_done (LSUP_Triple *spo)
{
    if (UNLIKELY (!spo)) return;

    LSUP_term_done (spo->s);
    LSUP_term_done (spo->p);
    LSUP_term_done (spo->o);
}


void
LSUP_striple_done (LSUP_SerTriple *sspo)
{
    if (UNLIKELY (!sspo)) return;

    LSUP_buffer_done (sspo->s);
    LSUP_buffer_done (sspo->p);
    LSUP_buffer_done (sspo->o);
}


void
LSUP_triple_free (LSUP_Triple *spo)
{
    if (UNLIKELY (!spo)) return;

    LSUP_term_free (spo->s);
    LSUP_term_free (spo->p);
    LSUP_term_free (spo->o);

    free (spo);
}


void
LSUP_striple_free (LSUP_SerTriple *sspo)
{
    if (UNLIKELY (!sspo)) return;

    LSUP_buffer_free (sspo->s);
    LSUP_buffer_free (sspo->p);
    LSUP_buffer_free (sspo->o);

    free (sspo);
}


/* Inline extern prototypes. */

LSUP_Key LSUP_striple_hash (const LSUP_SerTriple *strp);
LSUP_Key LSUP_triple_hash (const LSUP_Triple *trp);
